package com.eclipse.listener.scenario;

import java.util.ArrayList;
import java.util.Collection;

import com.eclipse.listener.scenario.task.Task;

public class Scenario {
	private String title;							// Title for the scenario. Ex: "Open Preferences".
	private boolean isFinished;						// True when the scenario is finished, false otherwise.
	private int currentTask;						// Keeps track of the currently activated task.
	private ArrayList<Task> tasks;					// The Tasks of the Scenario.
	private boolean isActivated;					// True when the scenario is activated, false otherwise.
	
	private Collection<ScenarioProgressListener> scenarioListeners;
	
	
	/**
	 * Constructor for the scenario, takes the title as a string parameter.
	 */
	public Scenario(String title){
		this.title = title;
		this.isFinished = false;
		this.isActivated = false;
		this.currentTask = 1;
		this.tasks = new ArrayList<Task>();
		this.scenarioListeners = new ArrayList<ScenarioProgressListener>();
	}
	
	/**
	 * Method for returning the title of the scenario.
	 * @return string title
	 */
	public String getTitle(){
		return this.title;
	}
	
	/**
	 * Method for showing the currently activated task.
	 * @return int currentTask
	 */
	public int showCurrentTask(){
		return this.currentTask;
	}
	
	/**
	 * Method for getting the ArrayList containing the Task-objects for the Scenario.
	 * @return ArrayList Task
	 */
	public ArrayList<Task> getTasks(){
		return this.tasks;
	}
	
	/**
	 * Method for adding a task to the ArrayList of Task-objects for the Scenario, and setting the Tasks-parentScenario to this Scenario.
	 * @param task
	 */
	public void addTask(Task task){
		task.setParentScenario(this);
		getTasks().add(task);
	}
	
	/**
	 * Method for checking if the scenario is finished.
	 * @return boolean: true if all the tasks of the scenario are finished, false otherwise.
	 */
	public boolean isFinished(){
		//If the Scenario is already finished, return true.
		if( this.isFinished == true ){
			return this.isFinished;	//true.
		}else{
			//Iterate the Tasks of the Scenario, find a Task which are not finished.
			boolean allTasksAreFinished = true;
			for(int i = 0; i < getTasks().size(); i++) {
				//If one of the Tasks is not finished, stop iterating.
				if( getTasks().get(i).isFinished() == false ){
					allTasksAreFinished = false;
					break;
				}
			}
			//If all of the Tasks of the Scenario was finished, return true.
			if( allTasksAreFinished == true ){
				this.isFinished = true;			
			}
			return this.isFinished;
		}		
	}
	
	/**
	 * Method is called when a Scenario is finished. Setting the isFinished variable of the Scenario to true.
	 */
	public void finishScenario(){
		this.isFinished = true;
		System.out.println("[Scenario]: Finish Scenario: " + this);
		fireScenarioProgress(this, true);	//Fire scenario is finished to listener.
		deactivateScenario();		
	}
	
	/**
	 * Method for checking if the scenario is activated.
	 * @return boolean, true if the scenario is activated, false otherwise.
	 */
	public boolean isActivated(){
		return this.isActivated;
	}
	
	/**
	 * Method for activating the scenario, setting the variable isActivated=true, and automatically activating the next non-finished Task in the Scenario.
	 */
	public void activateScenario(){
		this.isActivated = true;
		System.out.println("[Scenario]: Activate Scenario: " + this);
		activateNextTask();		
	}
	
	/**
	 * Method for activating the next non-finished Task in the Scenario. If the Scenario is completed, then finish the scenario.
	 */
	public void activateNextTask(){
		//If scenario is completed, then finish the scenario.
		if( isFinished() == true ){
			finishScenario();
		}else{
			Task task = findNextTask();
			if (task != null) {
				activateTask(task);
			}
		}		
	}
	
	/**
	 * Method for returning the next non-finished task.
	 * @return Task, or null if the tasks of the Scenario are finished.
	 */
	private Task findNextTask(){		
		for(Task task: tasks) {
			if( !task.isFinished() ){
				return task;
			}
		}		
		return null;
	}
	
	/**
	 * Method for activating the logic for a Task, and notifying the listeners that a Task has been activated.
	 * @param task
	 */
	private void activateTask(Task task){
		task.activateTask();
		fireScenarioProgress(task, false);
	}
	
	/**
	 * Method for notifying the listeners that a Task has been completed.
	 * @param task Task
	 */
	public void finishedTask(Task task){
		fireScenarioProgress(task, true);
	}

	/**
	 * Method for deactivating the scenario, setting the variable isActivated=false, deactivate any Tasks that is still activated.
	 */
	public void deactivateScenario(){
		this.isActivated = false;
		//If the user is exiting an incomplete Scenario, then deactivate any Tasks that is still activated.
		for(Task task : tasks) {
			if( task.isActivated() ){
				task.deactivateTask();
			}
		}		
		System.out.println("[Scenario]: Deactivate Scenario: " + this);
	}
	
	public String toString(){
		return "[Scenario]: Title: " +this.title+ ", isActivated: " +this.isActivated+ ", isFinished: " +this.isFinished+ "\nTasks: " +getTasks();
	}
	
	//Add scenario progress listener.
	public void addScenarioProgressListener(ScenarioProgressListener scenarioProgressListener){
		this.scenarioListeners.add(scenarioProgressListener);
	}
	//Remove scenario progress listener.
	public void removeScenarioProgressListener(ScenarioProgressListener scenarioProgressListener){
		this.scenarioListeners.remove(scenarioProgressListener);
	}
	
	/**
	 * Method for notifying the listeners about changes in the Scenario.
	 * @param task Task
	 * @param isTaskFinished boolean - true if task is finished, false otherwise.
	 */
	protected void fireScenarioProgress(Task task, boolean isTaskFinished){
		for (ScenarioProgressListener listener : scenarioListeners) {
			if(isTaskFinished){
				listener.taskCompleted(task);
			}else{
				listener.taskActivated(task);
			}
		}
	}
	/**
	 * Method for notifying the listeners about changes in the Scenario.
	 * @param scenario Scenario
	 * @param isScenarioFinished boolean - true if scenario is finished, false otherwise.
	 */
	protected void fireScenarioProgress(Scenario scenario, boolean isScenarioFinished){
		for (ScenarioProgressListener listener : scenarioListeners) {
			if(isScenarioFinished){
				listener.scenarioCompleted(scenario);
			}
		}
	}
	
	
}//end Scenario.