package com.eclipse.listener.scenario.task;

import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.IExecutionListener;
import org.eclipse.core.commands.NotHandledException;
import org.eclipse.core.commands.common.NotDefinedException;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Event;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.commands.ICommandService;

import com.eclipse.listener.scenario.task.autocomplete.AutoExecution;

/**
 * A subclass of Task.java that detects when a user is activating a command. A command could be a keyboard shortcut.
 * <br><br><b>ex:</b><i> CommandActivate("Content Assist", "Use keyboard to press CTRL + SPACE.", SWT.KeyDown)</i> 
 * <br><br>Content Assist, Paste, Copy.
 */
public class CommandActivate extends Task {
	//Listeners.
	private IExecutionListener commandActivateListener;	
	//CommandService.
	private ICommandService commandService;
	private int eventType;

	/**
	 * Constructor with commandTitle as a parameter, and SWT-eventType as parameter.
	 * @param commandTitle String - Ex: "Content Assist".
	 * @param eventType Integer - Ex: SWT.KeyDown, for a keyboard shortcut press.
	 */
	public CommandActivate(String commandTitle, int eventType) {
		super(commandTitle);
		this.eventType = eventType;
		createCommandActivateListener();
	}
	
	/**
	 * Constructor with commandTitle as a parameter. 
	 * @param commandTitle String - Ex: "Content Assist".
	 */
	public CommandActivate(String commandTitle) {
		this(commandTitle, SWT.None);
	}

	/**
	 * Constructor with commandTitle as a parameter, a description as a parameter, and SWT-eventType as parameter.
	 * @param commandTitle String - Ex: "Content Assist".
	 * @param description String - Ex: "Click CTRL + SPACE on the keyboard."
	 * @param eventType Integer - Ex: SWT.KeyDown, for a keyboard shortcut press.
	 */
	public CommandActivate(String commandTitle, String description, int eventType, AutoExecution autoExecuteTask) {
		super(commandTitle, description, autoExecuteTask);
		this.eventType = eventType;
		createCommandActivateListener();
	}	

	public void createCommandActivateListener() {
		System.out.println("[CommandActivate]: commandActivateListener for Task: " + getTitle());
		//IExecutionListener
		commandActivateListener = new IExecutionListener() {				
			@Override
			public void preExecute(String commandId, ExecutionEvent event) {
				System.out.println("[CommandActivate]: ExecutionEvent: " + event);
				try {
					System.out.println("[CommandActivate]: CommandName: "+event.getCommand().getName() );
					System.out.println("[CommandActivate]: CommandID: "+commandId );
					System.out.println("[CommandActivate]: CommandDescription: "+event.getCommand().getDescription() );
					System.out.println("Event trigger: "+event.getTrigger());
				} catch (NotDefinedException e) {
					System.out.println("[CommandActivate]: NotDefinedException: " + e);
					e.printStackTrace();
				}				
				
				//Check if current Task is "Content Assist", and command executed is Content Assist.
				if( getTitle().toLowerCase().equals("content assist") && commandId.equals("org.eclipse.ui.edit.text.contentAssist.proposals") ){
					
					if (eventType == SWT.None || (event.getTrigger() instanceof Event && ((Event) event.getTrigger()).type == eventType) || event.getTrigger()==null) {
						System.out.println("[CommandActivate]: Task completed! Used Content Assist");
						//Finish the task: isFinished=true, isActivated=false, removeListeners().
						finishTask();
					}
				}//content assist.						
				
			}//end preExecute.			
			@Override
			public void postExecuteSuccess(String commandId, Object returnValue) {}			
			@Override
			public void postExecuteFailure(String commandId,ExecutionException exception) {}			
			@Override
			public void notHandled(String commandId, NotHandledException exception) {}
		};//end commandActivateListener.		
		
	}//end createCommandActivateListener.

	@Override
	public void addListeners() {
		System.out.println("[CommandActivate]: Adding IExecutionListener: commandActivateListener for the Task: " + this );
		commandService = (ICommandService) PlatformUI.getWorkbench().getActiveWorkbenchWindow().getService(ICommandService.class);
		commandService.addExecutionListener(commandActivateListener);		//adding IExecutionListener.
	}//end addListeners.

	@Override
	public void removeListeners() {
		System.out.println("[CommandActivate]: Removing IExecutionListener: commandActivateListener for the Task: " + this );
		commandService.removeExecutionListener(commandActivateListener);	//removing IExecutionListener.
	}//end removeListeners.

}//end CommandActivate.