package com.eclipse.listener.scenario.task;

import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.ui.PlatformUI;

import com.eclipse.listener.scenario.task.autocomplete.AutoExecution;

/**
 * A subclass of Task.java 
 */
public abstract class SwtTask extends Task {
	/**
	 * Listener variable.
	 */
	private Listener swtListener;
	/**
	 * The SWT-EventType.
	 */
	private int eventType;

	/**
	 * Constructor with the title of a Task as parameter, and the SWT-event type as parameter.
	 * @param title String - Ex: "File" for a MenuBarClick.
	 * @param eventType Integer - Ex: "SWT.Show" for a MenuBarClick.
	 */
	public SwtTask(String title, int eventType) {
		super(title);
		this.eventType = eventType;
		createSwtListener();
	}
	
	/**
	 * Constructor with the title of a Task as parameter, a description of the Task as a parameter, and the SWT-event type as parameter.
	 * @param title String - Ex: "File" for a MenuBarClick.
	 * @param description String - Ex: "Click 'File' in the MenuBar." for a MenuBarClick.
	 * @param eventType Integer - Ex: "SWT.Show" for a MenuBarClick.
	 */
	public SwtTask(String title, String description, int eventType, AutoExecution autoExecuteTask) {
		super(title, description, autoExecuteTask);
		this.eventType = eventType;
		createSwtListener();
	}
	
	/**
	 * Method for creating the logic of the SWT-listener.
	 */
	public void createSwtListener() {
		System.out.println("["+getClass().getSimpleName()+"]: createSwtListener for Task: " + getTitle());
		swtListener = new Listener() {			
			@Override
			public void handleEvent(Event event) {
				if ( isTaskEvent(event) ) {
					System.out.println("[SwtTask]: COMPLETED Task: " + getTitle());
					//Finish the task: isFinished=true, isActivated=false, removeListeners().
					finishTask();
				}				
			}//handleEvent.
		};//swtListener.
	}//end createSwtListener.
	
	/**
	 * An abstract method for implementing the logic that verifies if an Event completes a Task.
	 * @param event Event - the event that occurred which the swtListener registered.
	 * @return boolean - true if the Event completed the Task, false otherwise.
	 */
	protected abstract boolean isTaskEvent(Event event);
	
	/**
	 * Method for adding the created swtListener to the display of the workbench.
	 */
	@Override
	public void addListeners() {
		System.out.println("["+getClass().getSimpleName()+"]: Adding Listener: swtListener(eventType "+this.eventType+") for the Task: " + this );
		PlatformUI.getWorkbench().getDisplay().addFilter( this.eventType, this.swtListener );	
	}//end addListeners.

	/**
	 * Method for removing the created swtListener from the display of the workbench.
	 */
	@Override
	public void removeListeners() {
		System.out.println("["+getClass().getSimpleName()+"]: Removing Listener: swtListener(eventType "+this.eventType+") for the Task: " + this );
		PlatformUI.getWorkbench().getDisplay().removeFilter( this.eventType, this.swtListener );
	}//end removeListeners.	

}//end SwtTask.