package com.eclipse.listener.scenario.task;

import org.eclipse.jface.dialogs.MessageDialog;

import com.eclipse.listener.scenario.Scenario;
import com.eclipse.listener.scenario.task.autocomplete.AutoExecution;

public abstract class Task {
	//Variables.
	private String title;
	private String description;
	private boolean isFinished;
	private boolean isActivated;
	private Scenario parentScenario;		//The Scenario the Task is connected to.
	private AutoExecution autoExecuteTask;	//The autoExecution-object for the Task. This object contains the code to programmatically execute the Task.
	
	/**
	 * Constructor. Takes a title of the task as a parameter.
	 * @param title String
	 */
	public Task(String title){
		setTitle(title);
		this.isFinished = false;
		this.isActivated = false;
		this.description = "";
	}
	
	/**
	 * Constructor. Takes a title of the task as a parameter, and a description as a parameter.
	 * @param title String
	 * @param description String 
	 */
	public Task(String title, String description, AutoExecution autoExecuteTask){
		setTitle(title);
		this.isFinished = false;
		this.isActivated = false;
		this.description = description;
		this.autoExecuteTask = autoExecuteTask;
	}
	
	/**
	 * Method for auto executing the Task.
	 */
	public void autoExecuteTask(){
		if(autoExecuteTask != null){	//Execute task.
			autoExecuteTask.execute();
		}else{
			//The task does not have an autoExecuteTask-object.
			MessageDialog.openError(null, "Error!", "The task, "+title+", does not have an autoExecuteTask-object");
		}
	}//end autoExecuteTask.
	
	/**
	 * Method that returns the boolean variable isFinished. True if the task is finished, false otherwise.
	 * @return boolean isFinished
	 */
	public boolean isFinished(){
		return this.isFinished;
	}
	
	/**
	 * Getter for the title of this Task.
	 * @return String title
	 */
	public String getTitle(){
		return this.title;
	}
	
	/**
	 * Getter for the description of this Task.
	 * @return String description.
	 */
	public String getDescription(){
		return this.description;
	}
	
	/**
	 * Method for setting the title of this Task.
	 * @param title String
	 */
	public void setTitle(String title){
		this.title = title;
	}
	
	/**
	 * Method for setting the description of this Task.
	 * @param description String
	 */
	public void setDescription(String description){
		this.description = description;
	}
	
	/**
	 * Method for getting the parent Scenario of the Task.
	 * @return Scenario - the parent Scenario of the Task.
	 */
	public Scenario getParentScenario(){
		return this.parentScenario;
	}
	
	/**
	 * Method for setting the parent Scenario of a Task.
	 * @param scenario
	 */
	public void setParentScenario(Scenario scenario){
		this.parentScenario = scenario;
	}
	
	/**
	 * Method for activating the Task, setting the isActivated to true and calling the method addListeners().
	 */
	public void activateTask(){		
		this.isActivated = true;
		System.out.println("[Task]: Activating Task: " + this);
		addListeners();
	}
	
	/**
	 * Method for deactivating the Task, setting the isActivated to false and calling the method removeListeners().
	 */
	public void deactivateTask(){
		this.isActivated = false;
		System.out.println("[Task]: Deactivate Task: " + this);
		removeListeners();		
	}
	
	/**
	 * Method which is marking the task as finished, deactivates the task(isActivated=false, removing listeners), and asks the Scenario to activate the next non-finished task.
	 */
	public void finishTask(){
		this.isFinished = true;
		System.out.println("[Task]: Finish Task: " + this);
		getParentScenario().finishedTask(this);
		deactivateTask();		
		getParentScenario().activateNextTask();
	}
	
	/**
	 * Method for checking if the Task is activated. 
	 * @return true if activated, and false if it is completed or deactivated.
	 */
	public boolean isActivated(){
		return this.isActivated;
	}
	
	/**
	 * Convenience method for removing any mnemonics from the given string. For example, removeMnemonics("&Open") will return "Open".
	 * @return string: the text sans mnemonics
	 */
	public static String removeMnemonics(String string){
		String result = org.eclipse.jface.action.Action.removeMnemonics( string );
		result = result.replace("...", "");
		return result.toLowerCase();
	}
	
	/**
	 * Method for adding the relevant listener/listeners to a Task. Ex: When the Task is activated.
	 */
	public abstract void addListeners();
	
	/**
	 * Method for removing the relevant listener/listeners to a Task. Ex: When the Task is deactivated.
	 */
	public abstract void removeListeners();
	
	/**
	 * ToString method.
	 * @return String.
	 */
	@Override
	public String toString(){
		return "[Task]: Title: " +this.title+ ", Description: " +this.description+ ", isActivated: " +this.isActivated+ ", isFinished: " +this.isFinished+ ", ParentScenario: " + getParentScenario().getTitle();
	} 
	
}//end Task.