package com.eclipse.listener.scenario.task;

import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;

import com.eclipse.listener.helpClasses.Methods;
import com.eclipse.listener.helpClasses.WorkspaceMethods;
import com.eclipse.listener.scenario.task.autocomplete.AutoExecution;

/**
 * A subclass of Task.java that detects when a user activates/opens an editor or a view in the workbench. 
 * <br><br><b>ex:</b> <i>WorkbenchPartActivate("Java File","Open/activate a random Java File").</i>
 * <br>This task will be completed when for instance "SampleJavaClass.java" is being opened/activated by the user.
 * <br><br><b>Possible types of String workbenchPart:</b>
 * <br>"Java File" - open a java file.
 * <br>"Editor" - open an editor.
 * <br>"View" - open a view. 
 */
public class WorkbenchPartActivate extends Task {
	//Listeners.
	public IPartListener workbenchPartActivateListener;

	/**
	 * Constructor with workbenchPart as a parameter.
	 * @param workbenchPart String
	 */
	public WorkbenchPartActivate(String workbenchPart) {
		super(workbenchPart);
		createWorkbenchPartActivateListener();
	}

	/**
	 * Constructor with workbenchPart as a parameter, and a description as a parameter.
	 * @param workbenchPart String
	 * @param description String
	 */
	public WorkbenchPartActivate(String workbenchPart, String description, AutoExecution autoExecuteTask) {
		super(workbenchPart, description, autoExecuteTask);
		createWorkbenchPartActivateListener();
	}
	

	public void createWorkbenchPartActivateListener() {
		System.out.println("[WorkbenchPartActivate]: createWorkbenchPartActivateListener for Task: " + getTitle());
		workbenchPartActivateListener = new IPartListener() {			
			@Override
			public void partOpened(IWorkbenchPart part) {
			}			
			@Override
			public void partDeactivated(IWorkbenchPart part) {				
			}			
			@Override
			public void partClosed(IWorkbenchPart part) {				
			}			
			@Override
			public void partBroughtToTop(IWorkbenchPart part) {				
			}			
			@Override
			public void partActivated(IWorkbenchPart part) {
				System.out.println("[WorkbenchPartActivate]: IWorkbenchPartActivated-Event: Title: " + part.getTitle() + ", TitleToolTip: " + part.getTitleToolTip()+ ", part: "+part);				
				//Check if Task title is "Java File". Ex: When a user opens/activates a Java File.
				if( getTitle().toLowerCase().equals("java file") ){
					//Check if the part that is activated is a Java File.
					try {
						String name = Methods.getCurrentlySelectedViewName();
						//Check if the currently opened/selected view is a java file.
						if( WorkspaceMethods.isAJavaFile(name) ){					
							System.out.println("[WorkbenchPartActivate]: Task completed! Opened a Java File: " + name);
							//Finish the task: isFinished=true, isActivated=false, removeListeners().
							finishTask();
						}
					} catch (Exception e) {
						System.out.println("[WorkbenchPartActivate]: isAJavaFile exception: " + e);
					}
				}//Java File.
				//Check if Task title is "Editor". Ex: When a user opens/activates an Editor.
				else if( getTitle().toLowerCase().equals("editor") ){
					//Check if the activated workbench part is an EDITOR.
					if( part instanceof IEditorPart){
						System.out.println("[WorkbenchPartActivate]: Task completed! Opened part is an Editor: " + part.getTitle());
						//Finish the task: isFinished=true, isActivated=false, removeListeners().
						finishTask();
					}
				}//editor.
				//Check if Task title is "View". Ex: When a user opens/activates a View.
				else if( getTitle().toLowerCase().equals("view") ){
					//Check if the activated workbench part is a VIEW.
					if( part instanceof IViewPart){
						System.out.println("[WorkbenchPartActivate]: Task completed! Opened part is a View: " + part.getTitle());
						//Finish the task: isFinished=true, isActivated=false, removeListeners().
						finishTask();
					}
				}//view.				
				
			}//end partActivated.
		};//end workbenchPartActivateListener.
		
	}//end createWorkbenchPartActivateListener.

	@Override
	public void addListeners() {
		System.out.println("[WorkbenchPartActivate]: Adding Listener: WorkbenchPartActivateListener for the Task: " + getTitle() );
		PlatformUI.getWorkbench().getActiveWorkbenchWindow().getPartService().addPartListener(workbenchPartActivateListener);
	}//end addListeners.

	@Override
	public void removeListeners() {
		System.out.println("[WorkbenchPartActivate]: Removing Listener: WorkbenchPartActivateListener for the Task: " + getTitle() );
		PlatformUI.getWorkbench().getActiveWorkbenchWindow().getPartService().removePartListener(workbenchPartActivateListener);
	}//end removeListeners.

}//end WorkbenchPartActivate.